class Movie < ActiveRecord::Base
  has_many :reviews
  has_many :showings
  has_many :theaters, :through => :showings
  named_scope :currently_playing, :include => :showings, :conditions => "showings.starts_on < current_date and showings.ends_on > current_date"
  
  validates_presence_of :name
  validates_associated :showings, :on => :update
  
  after_update :save_showings
  
  def self.search(search)
    currently_playing.find(:all, :conditions => ["name LIKE ?", "%#{search}%"], :order => 'name')
  end
  
  def average_stars
    update_attribute(:average_stars_cache, reviews.average(:stars)) if average_stars_cache.nil?
    average_stars_cache
  end
  
  def new_showing_attributes=(showing_attributes)
    showing_attributes.each do |attributes|
      showings.build(attributes)
    end
  end
  
  def existing_showing_attributes=(showing_attributes)
    showings.reject(&:new_record?).each do |showing|
      attributes = showing_attributes[showing.id.to_s]
      if attributes
        showing.attributes = attributes
      else
        showings.delete(showing)
      end
    end
  end
  
  private
  
  def save_showings
    if showings.loaded?
      showings.each do |showing|
        showing.save(false)
      end
    end
  end
end
