class Review < ActiveRecord::Base
  belongs_to :movie, :counter_cache => true
  
  named_scope :recent, lambda { |limit| {:order => "created_at DESC", :limit => limit} }
  named_scope :high_stars, :conditions => { :stars => [4, 5] }
  named_scope :low_stars, :conditions => { :stars => 1..3 }
  
  before_save :clear_movie_average_stars_cache
  before_destroy :clear_movie_average_stars_cache
  
  validates_presence_of :author_name
  validates_inclusion_of :stars, :in => 1..5, :allow_blank => true
  validates_exclusion_of :author_name, :in => 'Admin', :unless => :author_is_admin
  validates_presence_of :stars_or_review_content, :message => "must be filled in"
  
  attr_accessor :author_is_admin
  
  def self.filter_stars(filter)
    case filter
    when 'low' then low_stars.all
    when 'high' then high_stars.all
    else all
    end
  end
  
  def stars_or_review_content
    stars || content
  end
  
  private
  
  def clear_movie_average_stars_cache
    if movie_id_changed? && movie_id_was
      Movie.find(movie_id_was).update_attribute(:average_stars_cache, nil)
    end
    movie.update_attribute(:average_stars_cache, nil) if movie
  end
end
